@@ -43,7 +43,7 @@ class GrommunioDavBackend {
 		$userAgent = isset($_SERVER['HTTP_USER_AGENT']) ? $_SERVER['HTTP_USER_AGENT'] : 'unknown';
 		$this->session = mapi_logon_zarafa($user, $pass, MAPI_SERVER, null, null, 1, $gDavVersion, $userAgent);
 		if (!$this->session) {
-			$this->logger->info("Auth: ERROR - logon failed for user %s", $user);
+			$this->logger->info("Auth: ERROR - logon failed for user %s from IP %s", $user, $_SERVER['REMOTE_ADDR']);
 
 			return false;
 		}
@@ -137,16 +137,18 @@ class GrommunioDavBackend {
 			if ($row[PR_FOLDER_TYPE] == FOLDER_SEARCH) {
 				continue;
 			}
+			$folderId = $principalUri . ":" . bin2hex($row[PR_SOURCE_KEY]);
+			$syncToken = $this->GetCurrentSyncToken($folderId);
 
 			if (isset($row[PR_PARENT_ENTRYID], $storeprops[PR_IPM_WASTEBASKET_ENTRYID]) && $row[PR_PARENT_ENTRYID] == $storeprops[PR_IPM_WASTEBASKET_ENTRYID]) {
 				continue;
 			}
 
 			$folder = [
-				'id' => $principalUri . ":" . bin2hex($row[PR_SOURCE_KEY]),
+				'id' => $folderId,
 				'uri' => $row[PR_DISPLAY_NAME],
 				'principaluri' => $principalUri,
-				'{http://sabredav.org/ns}sync-token' => '0000000000',
+				'{http://sabredav.org/ns}sync-token' => $syncToken,
 				'{DAV:}displayname' => $row[PR_DISPLAY_NAME],
 				'{urn:ietf:params:xml:ns:caldav}calendar-description' => $row[PR_COMMENT],
 				'{http://calendarserver.org/ns/}getctag' => isset($row[PR_LOCAL_COMMIT_TIME_MAX]) ? strval($row[PR_LOCAL_COMMIT_TIME_MAX]) : '0000000000',
@@ -189,14 +191,12 @@ class GrommunioDavBackend {
 	 * @return null|array
 	 */
 	private function getRestrictionForFilters($filters, $storeId = null) {
-		$restrictions = [];
-		if (isset($filters['start'], $filters['end'], $storeId)) {
-			$this->logger->trace("getRestrictionForFilters - got start: %d and end: %d", $filters['start'], $filters['end']);
-			$subrestriction = $this->GetCalendarRestriction($storeId, $filters['start'], $filters['end']);
-			$restrictions[] = $subrestriction;
-		}
-		if (isset($filters['types'])) {
-			$this->logger->trace("getRestrictionForFilters - got types: %s", $filters['types']);
+		$hasTimerange = isset($filters['start'], $filters['end'], $storeId);
+		$hasTypes = isset($filters['types']) && is_array($filters['types']) && !empty($filters['types']);
+
+		// Fast path: only message-class filtering, no time-range.
+		if (!$hasTimerange && $hasTypes) {
+			$this->logger->trace("getRestrictionForFilters - types only: %s", $filters['types']);
 			$arr = [];
 			foreach ($filters['types'] as $filter) {
 				$arr[] = [RES_PROPERTY,
@@ -206,11 +206,48 @@ class GrommunioDavBackend {
 					],
 				];
 			}
-			$restrictions[] = [RES_OR, $arr];
+			$restriction = [RES_OR, $arr];
+			$this->logger->trace("getRestrictionForFilters - built: %s", simplifyRestriction($restriction));
+
+			return $restriction;
 		}
-		if (!empty($restrictions)) {
-			$restriction = [RES_AND, $restrictions];
-			$this->logger->trace("getRestrictionForFilters - got restriction: %s", simplifyRestriction($restriction));
+
+		// Time-range only (no explicit types) – interpret as events time-range.
+		if ($hasTimerange && !$hasTypes) {
+			$this->logger->trace("getRestrictionForFilters - timerange only start:%d end:%d", $filters['start'], $filters['end']);
+			$restriction = $this->GetCalendarRestriction($storeId, $filters['start'], $filters['end']);
+			$this->logger->trace("getRestrictionForFilters - built: %s", simplifyRestriction($restriction));
+
+			return $restriction;
+		}
+
+		// Both types and time-range. Apply date restriction to appointments only,
+		// and include other types (e.g., tasks) without date constraints.
+		if ($hasTimerange && $hasTypes) {
+			$this->logger->trace("getRestrictionForFilters - timerange + types start:%d end:%d types:%s", $filters['start'], $filters['end'], $filters['types']);
+			$orParts = [];
+
+			$dateRestriction = $this->GetCalendarRestriction($storeId, $filters['start'], $filters['end']);
+			foreach ($filters['types'] as $type) {
+				if ($type === 'IPM.Appointment') {
+					$orParts[] = [RES_AND, [
+						$dateRestriction,
+						[RES_PROPERTY, [RELOP => RELOP_EQ, ULPROPTAG => PR_MESSAGE_CLASS, VALUE => 'IPM.Appointment']],
+					]];
+				}
+				else {
+					// Other types (e.g., tasks) – no date constraint.
+					$orParts[] = [RES_PROPERTY, [RELOP => RELOP_EQ, ULPROPTAG => PR_MESSAGE_CLASS, VALUE => $type]];
+				}
+			}
+
+			// If no parts were added, return null.
+			if (empty($orParts)) {
+				return null;
+			}
+
+			$restriction = [RES_OR, $orParts];
+			$this->logger->trace("getRestrictionForFilters - built: %s", simplifyRestriction($restriction));
 
 			return $restriction;
 		}
@@ -375,13 +412,23 @@ class GrommunioDavBackend {
 			return $this->stores[$storename];
 		}
 
-		$this->stores[$storename] = $this->OpenMapiStore($storename);
-		if (!$this->stores[$storename]) {
+		$store = $this->OpenMapiStore($storename);
+		if (!$store) {
 			$this->logger->info("Auth: ERROR - unable to open store for %s (0x%08X)", $storename, mapi_last_hresult());
 
 			return false;
 		}
 
+		// g-dav#61: always use SMTP address (issue with altnames)
+		$storeProps = mapi_getprops($store, [PR_MAILBOX_OWNER_ENTRYID]);
+		$addressbook = $this->getAddressbook();
+		$mailuser = mapi_ab_openentry($addressbook, $storeProps[PR_MAILBOX_OWNER_ENTRYID]);
+		$smtpProps = mapi_getprops($mailuser, [PR_SMTP_ADDRESS]);
+		if (isset($smtpProps[PR_SMTP_ADDRESS])) {
+			$storename = $this->user = $smtpProps[PR_SMTP_ADDRESS];
+		}
+		$this->stores[$storename] = $store;
+
 		return $this->stores[$storename];
 	}
 
@@ -729,7 +776,7 @@ class GrommunioDavBackend {
 		}
 
 		$stream = mapi_stream_create();
-		if ($syncToken == null) {
+		if ($syncToken == null || $syncToken == '0000000000') {
 			mapi_stream_write($stream, hex2bin("0000000000000000"));
 		}
 		else {
@@ -819,6 +866,24 @@ class GrommunioDavBackend {
 		return $propsToSet;
 	}
 
+	/**
+	 * Returns the current sync-token for the folder if one was issued.
+	 *
+	 * @param string $folderId composite id in form principal:sourcekey
+	 *
+	 * @return string
+	*/
+	public function GetCurrentSyncToken($folderId) {
+		$arr = explode(':', $folderId, 2);
+		if (count($arr) < 2 || $arr[1] === '') {
+			return '0000000000';
+		}
+
+		$token = $this->syncstate->getCurrentToken($arr[1]);
+
+		return (!is_string($token) || $token === '') ? '0000000000' : $token;
+	}
+
 	/**
 	 * Checks whether the user is enabled for grommunio-dav.
 	 *
