@@ -1,8 +1,9 @@
 <?php
+
 /*
  * SPDX-License-Identifier: AGPL-3.0-only
  * SPDX-FileCopyrightText: Copyright 2016 - 2018 Kopano b.v.
- * SPDX-FileCopyrightText: Copyright 2020-2024 grommunio GmbH
+ * SPDX-FileCopyrightText: Copyright 2020 - 2025 grommunio GmbH
  *
  * Class for handling sync state.
  */
@@ -41,7 +42,7 @@ class GrommunioSyncState {
 	 * @param string $folderid
 	 * @param string $id
 	 *
-	 * @return string|null
+	 * @return null|string
 	 */
 	public function getState($folderid, $id) {
 		$query = "SELECT value FROM gdav_sync_state WHERE folderid = :folderid AND id = :id";
@@ -98,7 +99,7 @@ class GrommunioSyncState {
 	 * @param string $folderid
 	 * @param string $sourcekey
 	 *
-	 * @return string|null
+	 * @return null|string
 	 */
 	public function getAppttsref($folderid, $sourcekey) {
 		$query = "SELECT appttsref FROM gdav_sync_appttsref WHERE folderid = :folderid AND sourcekey = :sourcekey";
@@ -121,7 +122,7 @@ class GrommunioSyncState {
 	 * @param string $folderid
 	 * @param string $appttsref
 	 *
-	 * @return string|null
+	 * @return null|string
 	 */
 	public function getSourcekey($folderid, $appttsref) {
 		$query = "SELECT sourcekey FROM gdav_sync_appttsref WHERE folderid = :folderid AND appttsref = :appttsref";
@@ -136,4 +137,24 @@ class GrommunioSyncState {
 
 		return $result['sourcekey'];
 	}
+
+	/**
+	 * Fetch id(token) information for a folderId (e.g. calenderId).
+	 *
+	 * @param string $folderid
+	 *
+	 * @return null|string
+	 */
+	public function getCurrentToken($folderId) {
+		$query = "SELECT id FROM gdav_sync_state WHERE folderid = :folderid";
+		$statement = $this->db->prepare($query);
+		$statement->bindParam(":folderid", $folderid);
+		$statement->execute();
+		$result = $statement->fetch();
+		if (!$result) {
+			return null;
+		}
+
+		return $result['id'];
+	}
 }
