@@ -1,4 +1,5 @@
 <?php
+
 /*
  * SPDX-License-Identifier: AGPL-3.0-only
  * SPDX-FileCopyrightText: Copyright 2016 - 2018 Kopano b.v.
@@ -53,6 +54,7 @@ class MapiProps {
 			"rtfcompressed" => PR_RTF_COMPRESSED,
 			"html" => PR_HTML,
 			"rtfinsync" => PR_RTF_IN_SYNC,
+			"messageflags" => PR_MESSAGE_FLAGS,
 		];
 	}
 
@@ -66,6 +68,7 @@ class MapiProps {
 			"isrecurring" => false,
 			"meetingstatus" => olNonMeeting,
 			"responsestatus" => olResponseNone,
+			"messageflags" => MSGFLAG_READ,
 		];
 	}
 }
