@@ -1,8 +1,9 @@
 <?php
+
 /*
  * SPDX-License-Identifier: AGPL-3.0-only
  * SPDX-FileCopyrightText: Copyright 2016 - 2018 Kopano b.v.
- * SPDX-FileCopyrightText: Copyright 2020 grommunio GmbH
+ * SPDX-FileCopyrightText: Copyright 2020 - 2025 grommunio GmbH
  *
  * PHP wrapper class for ICS.
  */
@@ -17,11 +18,13 @@ class PHPWrapper {
 	private $added;
 	private $modified;
 	private $deleted;
+	private $syncstate;
+	private $folderid;
 
 	/**
 	 * Constructor.
 	 *
-	 * @param MAPIStore          $store
+	 * @param mixed              $store
 	 * @param GLogger            $logger
 	 * @param mixed              $props
 	 * @param string             $fileext
@@ -81,17 +84,15 @@ class PHPWrapper {
 	 * Imports a single message.
 	 *
 	 * @param array  $props
-	 * @param long   $flags
+	 * @param int    $flags
 	 * @param object $retmapimessage
 	 *
-	 * @return long
+	 * @return int
 	 */
 	public function ImportMessageChange($props, $flags, $retmapimessage) {
+		$entryid = $props[PR_ENTRYID] ?? null;
 		// if the entryid is not available, do the fallback to the sourcekey
-		if (isset($props[PR_ENTRYID])) {
-			$entryid = $props[PR_ENTRYID];
-		}
-		elseif (isset($props[PR_SOURCE_KEY], $props[PR_PARENT_SOURCE_KEY])) {
+		if (!$entryid && isset($props[PR_SOURCE_KEY], $props[PR_PARENT_SOURCE_KEY])) {
 			$entryid = mapi_msgstore_entryidfromsourcekey($this->store, $props[PR_PARENT_SOURCE_KEY], $props[PR_SOURCE_KEY]);
 		}
 		$mapimessage = mapi_msgstore_openentry($this->store, $entryid);
@@ -110,6 +111,7 @@ class PHPWrapper {
 			$this->logger->trace("got %s (PR_SOURCE_KEY), flags: %d", bin2hex($messageProps[PR_SOURCE_KEY]), $flags);
 			$url = bin2hex($messageProps[PR_SOURCE_KEY]);
 		}
+		$url = rawurlencode($url);
 
 		if ($flags == SYNC_NEW_MESSAGE) {
 			$this->added[] = $url . $this->fileext;
@@ -124,14 +126,14 @@ class PHPWrapper {
 	/**
 	 * Imports a list of messages to be deleted.
 	 *
-	 * @param long  $flags
+	 * @param int   $flags
 	 * @param array $sourcekeys array with sourcekeys
 	 */
 	public function ImportMessageDeletion($flags, $sourcekeys) {
 		foreach ($sourcekeys as $sourcekey) {
 			$this->logger->trace("got %s", bin2hex($sourcekey));
 			$appttsref = $this->syncstate->getAppttsref($this->folderid, bin2hex($sourcekey));
-			if ($appttsref != null) {
+			if ($appttsref !== null) {
 				$this->deleted[] = $appttsref . $this->fileext;
 			}
 			else {
@@ -141,20 +143,15 @@ class PHPWrapper {
 	}
 
 	/** Implement MAPI interface */
-	public function Config($stream, $flags = 0) {
-	}
+	public function Config($stream, $flags = 0) {}
 
-	public function GetLastError($hresult, $ulflags, &$lpmapierror) {
-	}
+	public function GetLastError($hresult, $ulflags, &$lpmapierror) {}
 
-	public function UpdateState($stream) {
-	}
+	public function UpdateState($stream) {}
 
-	public function ImportMessageMove($sourcekeysrcfolder, $sourcekeysrcmessage, $message, $sourcekeydestmessage, $changenumdestmessage) {
-	}
+	public function ImportMessageMove($sourcekeysrcfolder, $sourcekeysrcmessage, $message, $sourcekeydestmessage, $changenumdestmessage) {}
 
-	public function ImportPerUserReadStateChange($readstates) {
-	}
+	public function ImportPerUserReadStateChange($readstates) {}
 
 	public function ImportFolderChange($props) {
 		return 0;
